# coding=utf-8
import logging

from django.conf import settings
from django.db import models
from django.template import Engine
from django.utils.translation import ugettext_lazy as _

from emailtemplates.helpers import TemplateSourceLoader

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime

    now = datetime.now

logger = logging.getLogger(__name__)


class EmailTemplate(models.Model):
    """
    Model to store email template.
    """
    title = models.CharField(_(u'template'), max_length=255)
    subject = models.CharField(_(u'subject'), max_length=255, blank=True)
    content = models.TextField(_(u'content'))
    language = models.CharField(_(u'language'), max_length=10, choices=settings.LANGUAGES,
                                default=settings.LANGUAGE_CODE)
    created = models.DateTimeField(default=now)
    modified = models.DateTimeField(default=now)

    class Meta:
        unique_together = (('title', 'language'),)

    def __unicode__(self):
        return u'%s -> %s' % (self.title, self.language)

    def get_default_content(self):
        engine = Engine.get_default()
        loader = TemplateSourceLoader(engine)
        try:
            return loader.get_source(self.title)
        except Exception as e:
            logger.error('Error loading template %s. Details: %s ', self.title, e)
            return ''

    def save(self, *args, **kwargs):
        if not self.content:
            self.content = self.get_default_content()
        super(EmailTemplate, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-31 14:23
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(choices=[(b'accounts/activation.html', b'accounts/activation.html'), (b'project/introduction_project_mail_1d.html', b'project/introduction_project_mail_1d.html'), (b'project/notice_original_deletion_3days.html', b'project/notice_original_deletion_3days.html'), (b'accounts/password_reset_email.html', b'accounts/password_reset_email.html'), (b'accounts/email_verification.html', b'accounts/email_verification.html'), (b'share/invitation.html', b'share/invitation.html'), (b'orders/cancelled_payment.html', b'orders/cancelled_payment.html'), (b'project/new_template_available.html', b'project/new_template_available.html'), (b'supports/feedback_email.html', b'supports/feedback_email.html'), (b'invitations/invite_friend.html', b'invitations/invite_friend.html'), (b'project/introduction_project_mail_2d.html', b'project/introduction_project_mail_2d.html'), (b'share/invitation_subject.html', b'share/invitation_subject.html'), (b'orders/delivered.html', b'orders/delivered.html'), (b'project/project_action_notification.html', b'project/project_action_notification.html'), (b'project/notice_original_deletion_1week.html', b'project/notice_original_deletion_1week.html'), (b'accounts/welcome.html', b'accounts/welcome.html'), (b'project/download_zip.html', b'project/download_zip.html'), (b'orders/in_production.html', b'orders/in_production.html'), (b'project/notice_original_deletion_1month.html', b'project/notice_original_deletion_1month.html'), (b'accounts/introducing_email_4d.html', b'accounts/introducing_email_4d.html'), (b'orders/reorder_incentive_mail.html', b'orders/reorder_incentive_mail.html'), (b'project/project_action_like_comment_notification.html', b'project/project_action_like_comment_notification.html'), (b'accounts/introducing_email_2d.html', b'accounts/introducing_email_2d.html'), (b'project/package_expired.html', b'project/package_expired.html'), (b'project/package_upgrade.html', b'project/package_upgrade.html'), (b'project/introduction_project_mail_7d.html', b'project/introduction_project_mail_7d.html'), (b'subscriptions/subscription_email.html', b'subscriptions/subscription_email.html'), (b'accounts/introducing_email_3d.html', b'accounts/introducing_email_3d.html'), (b'accounts/introducing_email_1d.html', b'accounts/introducing_email_1d.html'), (b'supports/support_request.html', b'supports/support_request.html'), (b'supports/support_confirm.html', b'supports/support_confirm.html')], max_length=255, verbose_name='template')),
                ('subject', models.CharField(blank=True, max_length=255, verbose_name='subject')),
                ('content', models.TextField(verbose_name='content')),
                ('language', models.CharField(choices=[(b'de', b'German')], default=b'de', max_length=10, verbose_name='language')),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('modified', models.DateTimeField(default=django.utils.timezone.now)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='emailtemplate',
            unique_together=set([('title', 'language')]),
        ),
    ]



# coding=utf-8
from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.template import Template, Context
from django.views import View

from emailtemplates.models import EmailTemplate


class EmailPreviewView(View):
    def get_email_template(self):
        return get_object_or_404(EmailTemplate, pk=self.kwargs['pk'])

    def get_context_data(self):
        return {}

    def get(self, request, *args, **kwargs):
        email_template = self.get_email_template()
        email_content = Template(email_template.content)
        return HttpResponse(
            email_content.render(Context(self.get_context_data())),
            content_type='text/html; charset=utf-8'
        )


email_preview_view = staff_member_required(EmailPreviewView.as_view())



# coding=utf-8
import logging
from django.utils.translation import ugettext_lazy as _


logger = logging.getLogger(__name__)


class AlreadyRegistered(Exception):
    pass


class NotRegistered(Exception):
    pass


class RegistrationItem(object):

    def __init__(self, path, help_text=None, help_context=None):
        self.path = path
        self.help_text = help_text or u""
        self.help_context = help_context or {}

    def _context_key(self, key):
        return u"<b>{{ %s }}</b>" % key

    def context_description(self):
        help_text_item = lambda k, v: u"%s - %s" % (self._context_key(k), v) if v else u"%s" % self._context_key(k)
        return u"<br/>".join([help_text_item(k, v) for (k, v) in sorted(self.help_context.items())])

    def as_form_help_text(self):
        item_help_text = _(u"<b>USAGE: %s</b>") % self.help_text if self.help_text else u""
        item_help_context = _(u"<b>CONTEXT:</b><br/>%s") % self.context_description() if self.help_context else u""
        return u"<br/>".join((item_help_text, item_help_context))

    def as_form_choice(self):
        return self.path, self.path


class EmailTemplateRegistry(object):

    def __init__(self):
        self._registry = {}

    def register(self, path, help_text=None, help_context=None):
        """
        Registers email template.

        Example usage:
            email_templates.register('hello_template.html', help_text=u'Hello template',
                help_context={'username': u'Name of user in hello expression'})

        :param path: Template file path. It will become immutable registry lookup key.
        :param help_text: Help text to describe template in admin site
        :param help_context: Dictionary of possible keys used in the context and description of their content

        If an email template is already registered, this will raise AlreadyRegistered.
        """
        if path in self._registry:
            raise AlreadyRegistered('The template %s is already registered' % path)
        self._registry[path] = RegistrationItem(path, help_text, help_context)
        logger.debug("Registered email template %s", path)

    def is_registered(self, path):
        return path in self._registry

    def get_registration(self, path):
        """
        Returns registration item for specified path.

        If an email template is not registered, this will raise NotRegistered.
        """
        if not self.is_registered(path):
            raise NotRegistered("Email template not registered")
        return self._registry[path]

    def get_help_text(self, path):
        return self.get_registration(path).help_text

    def get_help_context(self, path):
        return self.get_registration(path).help_context

    def registration_items(self):
        return self._registry.values()

    def email_template_choices(self):
        """
        Returns list of choices that can be used in email template form field choices.
        """
        return [item.as_form_choice() for item in self.registration_items()]

    def get_form_help_text(self, path):
        """
        Returns text that can be used as form help text for creating email templates.
        """
        try:
            form_help_text = self.get_registration(path).as_form_help_text()
        except NotRegistered:
            form_help_text = u""
        return form_help_text


# Global object for singleton registry of email templates
email_templates = EmailTemplateRegistry()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-28 14:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('emailtemplates', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='title',
            field=models.CharField(choices=[(b'accounts/activation.html', b'accounts/activation.html'), (b'project/introduction_project_mail_1d.html', b'project/introduction_project_mail_1d.html'), (b'project/notice_original_deletion_3days.html', b'project/notice_original_deletion_3days.html'), (b'supports/feedback_email.html', b'supports/feedback_email.html'), (b'accounts/email_verification.html', b'accounts/email_verification.html'), (b'share/invitation.html', b'share/invitation.html'), (b'orders/cancelled_payment.html', b'orders/cancelled_payment.html'), (b'project/new_template_available.html', b'project/new_template_available.html'), (b'invitations/invite_friend.html', b'invitations/invite_friend.html'), (b'project/introduction_project_mail_2d.html', b'project/introduction_project_mail_2d.html'), (b'share/invitation_subject.html', b'share/invitation_subject.html'), (b'orders/delivered.html', b'orders/delivered.html'), (b'subscriptions/subscription_email.html', b'subscriptions/subscription_email.html'), (b'project/notice_original_deletion_1week.html', b'project/notice_original_deletion_1week.html'), (b'accounts/welcome.html', b'accounts/welcome.html'), (b'project/download_zip.html', b'project/download_zip.html'), (b'orders/in_production.html', b'orders/in_production.html'), (b'project/notice_original_deletion_1month.html', b'project/notice_original_deletion_1month.html'), (b'accounts/introducing_email_4d.html', b'accounts/introducing_email_4d.html'), (b'orders/reorder_incentive_mail.html', b'orders/reorder_incentive_mail.html'), (b'project/project_action_like_comment_notification.html', b'project/project_action_like_comment_notification.html'), (b'accounts/introducing_email_2d.html', b'accounts/introducing_email_2d.html'), (b'accounts/password_reset_email.html', b'accounts/password_reset_email.html'), (b'project/package_expired.html', b'project/package_expired.html'), (b'project/package_upgrade.html', b'project/package_upgrade.html'), (b'project/introduction_project_mail_7d.html', b'project/introduction_project_mail_7d.html'), (b'project/project_action_notification.html', b'project/project_action_notification.html'), (b'accounts/introducing_email_3d.html', b'accounts/introducing_email_3d.html'), (b'accounts/introducing_email_1d.html', b'accounts/introducing_email_1d.html'), (b'supports/support_request.html', b'supports/support_request.html'), (b'supports/support_confirm.html', b'supports/support_confirm.html')], max_length=255, verbose_name='template'),
        ),
    ]



import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

install_requires = [
    'Django>=1.8.8',
]

tests_require = [
    'mock',
]

setup(
    name='django-emailtemplates',
    version='1.0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A simple Django app to create emails based on database or filesystem templates.',
    long_description=README,
    url='https://github.com/deployed/django-emailtemplates',
    author='Wiktor Kolodziej',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=install_requires,
    tests_require=tests_require
)



# coding=utf-8
import mock
from django.conf import settings
from django.core import mail
from django.test import TestCase
from django.utils.html import escape
from mock import Mock

from emailtemplates.helpers import TemplateSourceLoader
from ..email import EmailFromTemplate
from ..email import logger as email_logger
from ..helpers import substr
from ..models import EmailTemplate
from ..registry import email_templates, NotRegistered, EmailTemplateRegistry


class CheckEmail(TestCase):
    def check_email_was_sent(self, eft, to):
        self.assertTrue(len(mail.outbox) > 0)
        msg = mail.outbox[0]
        self.assertTrue(settings.DEFAULT_FROM_EMAIL in msg.from_email)
        self.assertEqual(msg.content_subtype, 'html')
        self.assertEqual(msg.subject, eft.subject)
        self.assertEqual(msg.body, eft.message)
        self.assertTrue('Message-Id' in msg.message())
        self.assertEqual(msg.to, to)


class EmailFromTemplateTest(CheckEmail):
    def setUp(self):
        mail.outbox = []
        email_templates = EmailTemplateRegistry()
        email_logger.warning = Mock()

    def test_empty_object(self):
        eft = EmailFromTemplate(registry_validation=False)
        self.assertTrue(isinstance(eft, object))
        eft.render_message()
        to = ['to@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_with_empty_db_object(self):
        eft = EmailFromTemplate(registry_validation=False)
        eft.get_object()
        email_logger.warning.assert_has_calls(substr("Can't find EmailTemplate object in database"))
        email_logger.warning.assert_has_calls(substr("template in the filesystem, will use very default one"))
        eft.render_message()
        to = ['to@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_init_check_email_templates_registry(self):
        with self.assertRaises(NotRegistered):
            email_template = EmailFromTemplate("some_template.html")
        email_templates.register("some_template.html")
        email_template = EmailFromTemplate("some_template.html")
        self.assertTrue(email_templates.is_registered("some_template.html"))


class EmailFromTemplateWithFixturesTest(CheckEmail):
    def setUp(self):
        self.language = 'pl'
        email_templates = EmailTemplateRegistry()
        self.support_template = EmailTemplate.objects.create(
            language=self.language,
            title='support_respond.html',
            subject="Test",
            content="Support: {{ user_name }}"
        )
        mail.outbox = []
        email_logger.debug = Mock()

    def test_support_database_template(self):
        eft = EmailFromTemplate(name='support_respond.html', language=self.language, registry_validation=False)
        eft.get_object()
        email_logger.debug.assert_called_with(substr("Got template"))
        self.assertEqual(eft.template_source, 'database')
        eft.render_message()
        to = ['tester1@example.com', 'tester2@example.com']
        eft.send_email(to)
        self.check_email_was_sent(eft, to)

    def test_support_database_template_without_title(self):
        self.support_template.subject = ''
        self.support_template.save(update_fields=['subject'])
        eft = EmailFromTemplate(
            name='support_respond.html',
            subject='default email title',
            language=self.language,
            registry_validation=False
        )
        eft.get_object()
        self.assertEqual(eft.subject, 'default email title')

    def test_friends_invitation_no_database_or_filesystem_template(self):
        eft = EmailFromTemplate(registry_validation=False)
        eft.context = {'user_name': 'Alibaba',
                       'personal_message': "I'd like you te be site member!",
                       'landing_url': 'http://example.com/followers/612/'}
        eft.template = "{{ user_name }}, {{ personal_message }} {{ landing_url }}"
        eft.render_message()
        self.assertEqual(eft.message,
                         escape(u"Alibaba, I'd like you te be site member! http://example.com/followers/612/"))
        to = ['tester@example.com']
        self.assertEqual(eft.template_source, 'default')
        eft.send_email(to)
        self.check_email_was_sent(eft, to)


class EmailTemplateTest(TestCase):
    def setUp(self):
        self.default_content = '<h1>TEST DEFAULT CONTENT</h1>'
        self.email_template = EmailTemplate.objects.create(title='template-1.html')

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_get_default_content(self, mock_source):
        mock_source.return_value = self.default_content
        self.assertEqual(self.email_template.get_default_content(), self.default_content)

    @mock.patch.object(TemplateSourceLoader, 'get_source', mock.Mock(side_effect=Exception('error...')))
    def test_get_empty_default_content_if_error(self):
        self.assertEqual(self.email_template.get_default_content(), '')

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_save_default_content(self, mock_source):
        mock_source.return_value = self.default_content
        email_template = EmailTemplate.objects.create(title='template-2.html')
        self.assertEqual(email_template.content, self.default_content)

    @mock.patch.object(TemplateSourceLoader, 'get_source')
    def test_do_not_override_existing_content(self, mock_source):
        mock_source.return_value = self.default_content
        email_template = EmailTemplate.objects.create(title='template-2.html', content='<h1>New content</h1>')
        self.assertEqual(email_template.content, '<h1>New content</h1>')



# coding=utf-8
import logging

from django import forms
from django.core.exceptions import ValidationError
from django.template import Template
from django.template import TemplateSyntaxError
from django.utils.safestring import mark_safe

from emailtemplates.models import EmailTemplate
from emailtemplates.registry import email_templates

logger = logging.getLogger(__name__)


class EmailTemplateAdminForm(forms.ModelForm):
    title = forms.ChoiceField(choices=email_templates.email_template_choices())

    class Meta:
        model = EmailTemplate
        fields = [
            'title',
            'subject',
            'content',
            'language',
            'created',
            'modified',
        ]

    def __init__(self, *args, **kwargs):
        super(EmailTemplateAdminForm, self).__init__(*args, **kwargs)
        self.fields['title'].help_text = mark_safe(email_templates.get_form_help_text(self.initial.get('title')))
        if self.instance.pk:
            self.fields['title'].widget = forms.TextInput(attrs={'readonly': 'readonly', 'style': 'width:480px'})
        else:
            self.fields['content'].widget = forms.HiddenInput()
            self.fields['content'].required = False
            self.fields['subject'].widget = forms.HiddenInput()

    def clean_content(self):
        content = self.cleaned_data['content']
        try:
            Template(content)
        except TemplateSyntaxError as e:
            raise ValidationError(u"Syntax error in custom email template: %s" % e)
        return content
